{***************************************************************************
*                                                                          *
*        This program will print a text file sideways on the printer       *
*                                                                          *
*        Author - Ed Liss          Compuserve ID [71320,1642]              *
*                                                                          *
*        Copyright (C) 1985, Edward G Liss  All Rights Reserved            *
*        This program is released to the Public Domain with the            *
*        intention that it not be sold or used for profit.                 *
*                                                                          *
*                                                                          *
*        Sideways prints any ASCII text file on an EPSON RX-80 or          *
*        compatable printer.  This program has the following               *
*        hardware dependencies and limitations:                            *
*                                                                          *
*           - The character fonts are obtained from the IBM PC ROM.        *
*             If you are using a compatable, the video display char-       *
*             acters must be defined and used in the same manner.          *
*           - Maximum line length is 2048 characters.                      *
*           - Maximum number of lines per "page" is 48 with large          *
*             print and 96 with small print.                               *
*           - Your System must have enough memory to hold a "page"         *
*             of data.                                                     *
*                                                                          *
***************************************************************************}
{.PA}
{$R-,C-,U-}

program sideways;

const
    CHAR_PER_LINE     = 2048;    { Maximum characters per line }
    MAX_LINES         = 96;      { Maximum lines per "page" }
    SPACES_PER_LINE   = 2;       { 2/72th inch space between lines }
    SPACES_PER_LETTER = 8;       { DOT size of characters }

   {*** EPSON PRINTER CONSTANTS ****}
    set_vertical_size         = 'A';
    single_density_high_speed = 'K';
    double_density_high_speed = 'Y';
    double_density_low_speed  = 'L';
    reset_code                = '@';

type
    line_type = record
        line_len  :  integer;
        line_text :  array[1..char_per_line] of char;
    end; { of record }

var
    lines                       :  array[1..max_lines] of ^Line_type;
    file_name                   :  string[65];
    line_count                  :  byte;
    text_in                     :  file;
    text_buff                   :  array[1..128] of char;
    text_next                   :  byte;
    e_o_f                       :  boolean;
    a_char,
    density,
    char_size,
    graphics_mode               :  char;
    max_lines_at_density        :  integer;

    num_blocks                  :  real;
    line_in                     :  line_type;

    heap_top                    :  ^Integer;
    i                           :  integer;
    ROM                         :  array[0..32000] of char
                                   absolute $F000:$FA6E; { PC-ONLY }

{          Check_for_key_stroke give the user the option
                 of aborting the print at any time             }

procedure Check_for_key_stroke;

begin
    if keypressed then
    begin
        while keypressed do
            read(kbd,a_char);       { flush the key board buffer }
        writeln('Press ^C To Abort, Any Other Key To Continue');
        read(kbd,a_char);
        if a_char = ^C then
        begin
            close(text_in);
            writeln(lst,#27,reset_code);
            halt;
        end;
    end;
end;

{     get_line reads the lines from the disk file.  Simple read into a string
       cannot be used since strings are limited to 255 characters.  This
       routine will input a line up to char_per_line characters               }

procedure get_line;

var  End_of_line  :  boolean;

begin
    end_of_line := false;
    line_in.line_len := 0;
    repeat
        text_next := text_next + 1;
        if text_next > 128 then
        begin
            if num_blocks > 0.0 then
            begin
                blockread(text_in,text_buff,1);
                num_blocks := num_blocks - 1.0;
            end
            else
                text_buff[1] := ^Z;              { force standard EOF }
            text_next := 1;
        end;
        if text_buff[text_next] <> ^J then       { ignore line feeds - ^J }
        begin
            end_of_line := text_buff[text_next] in [^M,^L,^Z];
            if not end_of_line then
            begin
                line_in.line_len := line_in.line_len + 1;
                line_in.line_text[line_in.line_len] := text_buff[text_next];
            end;
        end;
    until end_of_line;
    e_o_f := text_buff[text_next] = ^Z;
end; { of get_line }

     {  print_page examines the page in memory and prints it out  }

procedure print_page;

var  i,j  :  integer;
     first_non_blank_found : boolean;
     first_non_blank,
     longest_line          :  integer;
     line,
     col                   :  integer;
     ndx                   :  integer;
     print_char            :  integer;
     line_width            :  array[0..2] of integer;

begin
    longest_line := 0;                               { determine longest line }
    for j := 1 to line_count do                      { for print optimization }
    begin
        if lines[j]^.line_len > longest_line then
            longest_line := lines[j]^.line_len;
    end;
    check_for_key_stroke;
    for col := 1 to longest_line do
    begin
        first_non_blank := 0;                          { find first non blank }
        first_non_blank_found := false;                { in a column of chars }
        repeat                                         { to avoid printing    }
            first_non_blank := first_non_blank + 1;            { extra blanks }
            if col > lines[first_non_blank]^.line_len then
            else
               if lines[first_non_blank]^.line_text[col] <> ' ' then
                   first_non_blank_found := true;
            first_non_blank_found := first_non_blank_found  or
                 (first_non_blank = line_count);
        until first_non_blank_found;
                                          { compute the number of bits that   }
                                          { the current column will contain   }
                                          { so the printer can be set up      }

        line_width[0] := (line_count - first_non_blank + 1)
                               * (8 + spaces_per_line);
        if (density = 'D') and (char_size = 'L') then
            line_width[0] := line_width[0] * 2;
        line_width[1] := line_width[0] div 256;
        line_width[2] := line_width[0] mod 256;
        write(lst,#27,set_vertical_size,chr(spaces_per_letter),
                  #27,graphics_mode,chr(line_width[2]), chr(line_width[1]));
        for line := line_count downto first_non_blank do
        begin
            if col > lines[line]^.line_len then
                print_char := 32  { space }
            else
                print_char := ord(lines[line]^.line_text[col]);
            print_char := print_char * 8;
            if (density = 'D') and (char_size = 'L') then
            begin
                for ndx := 7 downto 0 do
                    write(lst,rom[print_char + ndx],rom[print_char + ndx]);
                for ndx := 1 to spaces_per_line do
                    write(lst,chr(0),chr(0));
            end
            else
            begin
                for ndx := 7 downto 0 do
                    write(lst,rom[print_char + ndx]);
                for ndx := 1 to spaces_per_line do
                    write(lst,chr(0));
            end;
        end;
        writeln(lst);
        check_for_key_stroke;
    end;
    writeln(lst,^L);
    release(heap_top);
    mark(heap_top);
end;

Procedure Determine_variables;

begin
    repeat
        write('Large or Small Characters? ');
        readln(char_size);
        char_size := upcase(char_size);
    until char_size in ['L','S'];

    repeat
        write('Single or Double Density? ');
        readln(density);
        density := upcase(density);
    until density in ['S','D'];

    if char_size = 'L' then
       if density = 'S' then
       begin
          graphics_mode := single_density_high_speed;
          max_lines_at_density := max_lines div 2;
       end
       else
       begin
          graphics_mode := double_density_low_speed;
          max_lines_at_density := max_lines div 2;
       end
    else
       if density = 'S' then
       begin
          graphics_mode := double_density_high_speed;
          max_lines_at_density := max_lines;
       end
       else
       begin
          graphics_mode := double_density_low_speed;
          max_lines_at_density := max_lines;
       end;
end; { of determine_variables }

begin   { main program }
    mark(heap_top);
    file_name := '';
    char_size := ' ';
    density := ' ';
    writeln('Enter Name Of Text File to Print Sideways: [drive:][\path]file.ext');
    readln(file_name);

    assign(text_in,file_name);
  {$I-}
    reset(text_in);
  {$I+}
    if ioresult <> 0 then
    begin
        writeln('Unable to Open File ',file_name,^G);
        halt;
    end;

    determine_variables;

    writeln('Press Any Key To Pause');

    num_blocks := longfilesize(text_in);
    line_count := 0;
    text_next := 128;
    repeat
        get_line;
        line_count := line_count + 1;
        getmem(lines[line_count],line_in.line_len+2);
        move(line_in.line_len,lines[line_count]^.line_len,line_in.line_len+2);
        if line_count = max_lines_at_density then
        begin
            print_page;
            line_count := 0;
        end;
    until e_o_f;

 {**** Create Blank Lines To Fill Up The Page ****}

    if line_count > 0 then
    begin
        while line_count < max_lines_at_density do
        begin
            line_count := line_count + 1;
            line_in.line_len := 0;
            getmem(lines[line_count],2);
            move(line_in.line_len,lines[line_count]^.line_len,2);
        end;
        print_page;
    end;

    close(text_in);
    release(heap_top);
    writeln(lst,#27,reset_code);
end.